@@ -1,6 +1,20 @@
-(* $I1: Unison file synchronizer: src/ubase/util.ml $ *)
-(* $I2: Last modified by vouillon on Mon, 14 Jun 2004 16:38:56 -0400 $ *)
-(* $I3: Copyright 1999-2004 (see COPYING for details) $ *)
+(* Unison file synchronizer: src/ubase/util.ml *)
+(* Copyright 1999-2012, Benjamin C. Pierce 
+
+    This program is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*)
+
 
 (*****************************************************************************)
 (*                        CASE INSENSITIVE COMPARISON                        *)
@@ -23,17 +37,8 @@ let nocase_eq a b = (0 = (nocase_cmp a b))
 (*                        PRE-BUILT MAP AND SET MODULES                      *)
 (*****************************************************************************)
 
-module StringMap =
-  Map.Make(struct
-    type t = string
-    let compare = compare
-  end)
-
-module StringSet =
-  Set.Make(struct
-    type t = string
-    let compare = compare
-  end)
+module StringMap = Map.Make (String)
+module StringSet = Set.Make (String)
 
 let stringSetFromList l =
   Safelist.fold_right StringSet.add l StringSet.empty
@@ -61,6 +66,23 @@ let msg f =
 
 let msg : ('a, out_channel, unit) format -> 'a = msg
 
+(* ------------- Formatting stuff --------------- *)
+
+let curr_formatter = ref Format.std_formatter
+
+let format f = Format.fprintf (!curr_formatter) f
+let format : ('a, Format.formatter, unit) format -> 'a = format
+
+let format_to_string f =
+  let old_formatter = !curr_formatter in
+  curr_formatter := Format.str_formatter;
+  f ();
+  let s = Format.flush_str_formatter () in
+  curr_formatter := old_formatter;
+  s
+    
+let flush () = Format.pp_print_flush (!curr_formatter) ()
+
 (*****************************************************************************)
 (*                  GLOBAL DEBUGGING SWITCH                                  *)
 (*****************************************************************************)
@@ -103,7 +125,11 @@ let encodeException m kind e =
     Unix.Unix_error(err,fnname,param) ->
       let s =   "Error in " ^ m ^ ":\n"
               ^ (Unix.error_message err)
-              ^ " [" ^ fnname ^ "(" ^ param ^ ")]" in
+              ^ " [" ^ fnname ^ "(" ^ param ^ ")]%s" ^
+              (match err with
+                 Unix.EUNKNOWNERR n -> Format.sprintf " (code %d)" n
+               | _                  -> "")
+      in
       debug "exn"
         (fun() -> msg "Converting a Unix error to %s:\n%s\n" kindStr s);
       reraise s
@@ -120,6 +146,12 @@ let encodeException m kind e =
       debug "exn"
         (fun() -> msg "Converting a Not_found to %s:\n%s\n" kindStr s);
       reraise s
+  | Invalid_argument a ->
+      let s = "Invalid_argument("^a^") raised in " ^ m
+              ^ " (this indicates a bug!)" in
+      debug "exn"
+        (fun() -> msg "Converting an Invalid_argument to %s:\n%s\n" kindStr s);
+      reraise s
   | Sys_error(s) ->
       let s = "Error in " ^ m ^ ":\n" ^ s in
       debug "exn"
@@ -206,16 +238,29 @@ let ignoreTransientErrors thunk =
   with
     Transient(s) -> ()
 
+let printException e =
+  try
+    raise e
+  with
+    Transient s -> s
+  | Fatal s -> s
+  | e -> Printexc.to_string e
+
 (* Safe version of Unix getenv -- raises a comprehensible error message if
    called with an env variable that doesn't exist                            *)
 let safeGetenv var =
   convertUnixErrorsToFatal
     "querying environment"
     (fun () ->
-       try Unix.getenv var
+       try System.getenv var
        with Not_found ->
          raise (Fatal ("Environment variable " ^ var ^ " not found")))
 
+let process_status_to_string = function
+    Unix.WEXITED i   -> Printf.sprintf "Exited with status %d" i
+  | Unix.WSIGNALED i -> Printf.sprintf "Killed by signal %d" i
+  | Unix.WSTOPPED i  -> Printf.sprintf "Stopped by signal %d" i
+
 (*****************************************************************************)
 (*                         OS TYPE                                           *)
 (*****************************************************************************)
@@ -224,7 +269,7 @@ let osType =
   match Sys.os_type with
     "Win32" | "Cygwin" -> `Win32
   | "Unix"             -> `Unix
-  | other              -> raise (Fatal ("Unkown OS: " ^ other))
+  | other              -> raise (Fatal ("Unknown OS: " ^ other))
 
 let isCygwin = (Sys.os_type = "Cygwin")
 
@@ -244,14 +289,28 @@ let time () =
   convertUnixErrorsToTransient "time" Unix.time
 
 let time2string timef =
-  let time = localtime timef in
-  Printf.sprintf
-    "%2d:%.2d on %2d %3s, %4d"
-    time.Unix.tm_hour
-    time.Unix.tm_min
-    time.Unix.tm_mday
-    (monthname time.Unix.tm_mon)
-    (time.Unix.tm_year + 1900)
+  try
+    let time = localtime timef in
+(* Old-style:
+    Printf.sprintf
+      "%2d:%.2d:%.2d on %2d %3s, %4d"
+      time.Unix.tm_hour
+      time.Unix.tm_min
+      time.Unix.tm_sec
+      time.Unix.tm_mday
+      (monthname time.Unix.tm_mon)
+      (time.Unix.tm_year + 1900)
+*)
+    Printf.sprintf
+      "%4d-%02d-%02d at %2d:%.2d:%.2d"
+      (time.Unix.tm_year + 1900)
+      (time.Unix.tm_mon + 1)
+      time.Unix.tm_mday
+      time.Unix.tm_hour
+      time.Unix.tm_min
+      time.Unix.tm_sec
+  with Transient _ ->
+    "(invalid date)"
 
 let percentageOfTotal current total =
   (int_of_float ((float current) *. 100.0 /. (float total)))
@@ -272,8 +331,8 @@ let option2string (prt: 'a -> string) = function
 
 let truncateString string length =
   let actualLength = String.length string in
-  if actualLength <= length then
-    string^(String.make (length - actualLength) ' ')
+  if actualLength <= length then string^(String.make (length - actualLength) ' ')
+  else if actualLength < 3 then string
   else (String.sub string 0 (length - 3))^ "..."
 
 let findsubstring s1 s2 =
@@ -294,6 +353,11 @@ let rec replacesubstring s fromstring tostring =
       let after = String.sub s afterpos ((String.length s) - afterpos) in
       before ^ tostring ^ (replacesubstring after fromstring tostring)
 
+let replacesubstrings s pairs =
+  Safelist.fold_left 
+    (fun s' (froms,tos) -> replacesubstring s' froms tos)
+    s pairs
+
 let startswith s1 s2 =
   let l1 = String.length s1 in
   let l2 = String.length s2 in
@@ -318,6 +382,12 @@ let endswith s1 s2 =
 let concatmap sep f l =
   String.concat sep (Safelist.map f l)
 
+let removeTrailingCR s =
+  let l = String.length s in
+  if l = 0 || s.[l - 1] <> '\r' then s else
+  String.sub s 0 (l - 1)
+
+(* FIX: quadratic! *)
 let rec trimWhitespace s =
   let l = String.length s in
   if l=0 then s
@@ -354,20 +424,26 @@ let padto n s = s ^ (String.make (max 0 (n - String.length s)) ' ')
 (*              Building pathnames in the user's home dir                    *)
 (*****************************************************************************)
 
-let fileInHomeDir n =
-  match osType with
-    `Win32 ->
-      let dirString =
-        try Unix.getenv "USERPROFILE" (* Windows NT/2K *)
-        with Not_found ->
-        try Unix.getenv "HOME" (* Windows 9x with Cygwin HOME set *)
-        with Not_found ->
-        try Unix.getenv "UNISON" (* Use UNISON dir if none of
-                                    the above are set *)
-        with Not_found -> "c:/" (* Default *) in
-      Filename.concat dirString n
-  | `Unix ->
-      Filename.concat (safeGetenv "HOME") n
+let homeDir () =
+  System.fspathFromString
+    (if (osType = `Unix) || isCygwin then
+       safeGetenv "HOME"
+     else if osType = `Win32 then
+(*We don't want the behavior of Unison to depends on whether it is run
+  from a Cygwin shell (where HOME is set) or in any other way (where
+  HOME is usually not set)
+       try System.getenv "HOME" (* Windows 9x with Cygwin HOME set *)
+       with Not_found ->
+*)
+       try System.getenv "USERPROFILE" (* Windows NT/2K standard *)
+       with Not_found ->
+       try System.getenv "UNISON" (* Use UNISON dir if it is set *)
+       with Not_found ->
+       "c:/" (* Default *)
+     else
+       assert false (* osType can't be anything else *))
+
+let fileInHomeDir n = System.fspathConcat (homeDir ()) n
 
 (*****************************************************************************)
 (*           "Upcall" for building pathnames in the .unison dir              *)
