@@ -1,18 +1,32 @@
-(* $I1: Unison file synchronizer: src/uutil.ml $ *)
-(* $I2: Last modified by vouillon on Tue, 31 Aug 2004 11:33:38 -0400 $ *)
-(* $I3: Copyright 1999-2004 (see COPYING for details) $ *)
+(* Unison file synchronizer: src/uutil.ml *)
+(* Copyright 1999-2012, Benjamin C. Pierce 
+
+    This program is free software: you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation, either version 3 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*)
+
 
 (*****************************************************************************)
 (*                      Unison name and version                              *)
 (*****************************************************************************)
 
-(* $Format: "let myName = \"$Project$\""$ *)
-let myName = "unison"
+let myName = ProjectInfo.myName
 
-let versionprefix = "2."
+let myVersion = ProjectInfo.myVersion
 
-(* $Format: "let myVersion = versionprefix ^ \"$ProjectVersion$\""$ *)
-let myVersion = versionprefix ^ "10.2"
+let myMajorVersion = ProjectInfo.myMajorVersion
+
+let myNameAndVersion = myName ^ " " ^ myVersion
 
 (*****************************************************************************)
 (*                             HASHING                                       *)
@@ -30,6 +44,7 @@ module type FILESIZE = sig
   val dummy : t
   val add : t -> t -> t
   val sub : t -> t -> t
+  val ofFloat : float -> t
   val toFloat : t -> float
   val toString : t -> string
   val ofInt : int -> t
@@ -43,10 +58,11 @@ end
 
 module Filesize : FILESIZE = struct
   type t = int64
-  let zero = Int64.zero
-  let dummy = Int64.minus_one
+  let zero = 0L
+  let dummy = -1L
   let add = Int64.add
   let sub = Int64.sub
+  let ofFloat = Int64.of_float
   let toFloat = Int64.to_float
   let toString = Int64.to_string
   let ofInt x = Int64.of_int x
@@ -72,6 +88,7 @@ module File =
     let dummy = -1
     let ofLine l = l
     let toLine l = assert (l <> dummy); l
+    let toString l = if l=dummy then "<dummy>" else string_of_int l
   end
 
 let progressPrinter = ref (fun _ _ _ -> ())
@@ -79,46 +96,71 @@ let setProgressPrinter p = progressPrinter := p
 let showProgress i bytes ch =
   if i <> File.dummy then !progressPrinter i bytes ch
 
+let statusPrinter = ref None
+let setUpdateStatusPrinter p = statusPrinter := p
+let showUpdateStatus path =
+  match !statusPrinter with
+    Some f -> f path
+  | None   -> Trace.statusDetail path
+
 (*****************************************************************************)
 (*               Copy bytes from one file_desc to another                    *)
 (*****************************************************************************)
 
-let bufsize = 10000
+let bufsize = 16384
 let bufsizeFS = Filesize.ofInt bufsize
 let buf = String.create bufsize
 
 let readWrite source target notify =
+  let len = ref 0 in
   let rec read () =
-    let n = Unix.read source buf 0 bufsize in
-    if n>0 then begin
-      let rec write pos =
-        let w = Unix.write target buf pos (n-pos) in
-        notify w;
-        if (pos+w)<>n then write (pos+w)
-      in
-      write 0;
-      read()
-    end
+    let n = input source buf 0 bufsize in
+    if n > 0 then begin
+      output target buf 0 n;
+      len := !len + n;
+      if !len > 100 * 1024 then begin
+        notify !len;
+        len := 0
+      end;
+      read ()
+    end else if !len > 0 then
+      notify !len
   in
-    Util.convertUnixErrorsToTransient "readWrite"
-      read
+  Util.convertUnixErrorsToTransient "readWrite" read
 
 let readWriteBounded source target len notify =
+  let l = ref 0 in
   let rec read len =
     if len > Filesize.zero then begin
       let n =
-        Unix.read source buf 0
+        input source buf 0
           (if len > bufsizeFS then bufsize else Filesize.toInt len)
       in
       if n > 0 then begin
-        let rec write pos =
-          let w = Unix.write target buf pos (n - pos) in
-          notify w;
-          if (pos + w) <> n then write (pos + w)
-        in
-        write 0;
+        let _ = output target buf 0 n in
+        l := !l + n;
+        if !l >= 100 * 1024 then begin
+          notify !l;
+          l := 0
+        end;
         read (Filesize.sub len (Filesize.ofInt n))
-      end
-    end
+      end else if !l > 0 then
+        notify !l
+    end else if !l > 0 then
+      notify !l
   in
-  Util.convertUnixErrorsToTransient "readWrite" (fun () -> read len)
+  Util.convertUnixErrorsToTransient "readWriteBounded" (fun () -> read len)
+
+(*****************************************************************************)
+(*                      ESCAPING SHELL PARAMETERS                            *)
+(*****************************************************************************)
+
+(* Using single quotes is simpler under Unix but they are not accepted
+   by the Windows shell.  Double quotes without further quoting is
+   sufficient with Windows as filenames are not allowed to contain
+   double quotes. *)
+let quotes s =
+  if Util.osType = `Win32 && not Util.isCygwin then
+    "\"" ^ s ^ "\""
+  else
+    "'" ^ Util.replacesubstring s "'" "'\\''" ^ "'"
